/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.constraint;

import static org.fortiss.tooling.kernel.constraint.ConstraintMessage.createConsistencyIdViolation;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getChildrenWithType;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.fortiss.tooling.kernel.extension.IConstraintChecker;
import org.fortiss.tooling.kernel.extension.base.ConstraintCheckerBase;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.IIdLabeled;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * {@link IConstraintChecker} to check whether the IDs provided by {@link IIdLabeled} is identical
 * to the XMI ID stored in the corresponding {@link XMLResource}.
 * 
 * While this consistency of the IDs should be ensured by of the kernel (see {@code ModelContext}),
 * this constraint checker is an additional safety layer.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: B88B5832B0BFA99359A3DA7F20BBBCBE
 */
public class IdConsistencyChecker extends ConstraintCheckerBase<IIdLabeled> {

	/** {@inheritDoc} */
	@Override
	public List<IdConsistencyConstraintViolation<IIdLabeled>> apply(IIdLabeled element) {
		ArrayList<IdConsistencyConstraintViolation<IIdLabeled>> result =
				new ArrayList<IdConsistencyConstraintViolation<IIdLabeled>>();

		// Get project root element
		EObject root = element;
		while(root.eContainer() != null) {
			root = root.eContainer();
		}

		ITopLevelElement topLevelElement = IPersistencyService.INSTANCE.getTopLevelElementFor(root);
		if(topLevelElement != null) {
			for(IIdLabeled current : getChildrenWithType(root, IIdLabeled.class)) {
				String persistedId = topLevelElement.getId(current);
				if(!(new Integer(current.getId()).toString()).equals(persistedId)) {
					result.add(createConsistencyIdViolation(current, persistedId));
				}
			}
		}
		return result;
	}
}
