/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.constraint;

import static org.fortiss.tooling.kernel.constraint.ConstraintMessage.createDuplicateIdViolation;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.getChildrenWithType;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.fortiss.tooling.kernel.extension.IConstraintChecker;
import org.fortiss.tooling.kernel.extension.base.ConstraintCheckerBase;
import org.fortiss.tooling.kernel.model.IIdLabeled;

/**
 * {@link IConstraintChecker} to check whether all {@link IIdLabeled} elements actually have unique
 * IDs.
 * 
 * While the uniqueness of IDs should be ensured by the assignment strategy of the kernel, and
 * {@code ModelContext} eliminates duplicated IDs when loading and saving models, this constraint
 * checker is an additional safety layer.
 * 
 * This is because in persisted model resources, the encoding of {@link EReference} is based on IDs.
 * Hence, model resources with duplicated IDs in referenced model elements are invalid and cannot be
 * loaded.
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: A57025B59A7DB40046D9DD46EDDD5315
 */
public class IdUniquenessChecker extends ConstraintCheckerBase<IIdLabeled> {

	/** Map to track ID -> element relationship (within a single project). */
	private class IdToElementMap extends HashMap<Integer, IIdLabeled> {
		// Nothing to do
	}

	/** Map: project root element -> {@link IdToElementMap}. */
	private Map<EObject, IdToElementMap> idToElementMaps = new HashMap<EObject, IdToElementMap>();

	/** {@inheritDoc} */
	@Override
	public List<IdUniquenessConstraintViolation<IIdLabeled>> apply(IIdLabeled element) {
		ArrayList<IdUniquenessConstraintViolation<IIdLabeled>> result =
				new ArrayList<IdUniquenessConstraintViolation<IIdLabeled>>();

		// Get project root element
		EObject root = element;
		while(root.eContainer() != null) {
			root = root.eContainer();
		}

		IdToElementMap idToElementMap = idToElementMaps.get(root);

		// If required, populate map to track ID -> Element relationship
		if(idToElementMap == null) {
			idToElementMap = new IdToElementMap();
			idToElementMaps.put(root, idToElementMap);

			for(IIdLabeled current : getChildrenWithType(root, IIdLabeled.class)) {
				// In case there are duplicates, the first occurrence is considered to be legitimate
				// and is hence kept in the map
				if(!idToElementMap.containsKey(current.getId())) {
					idToElementMap.put(current.getId(), current);
				}
			}
		}

		if(idToElementMap.containsKey(element.getId())) {
			IIdLabeled knownElement = idToElementMap.get(element.getId());
			if(knownElement != element) {
				result.add(createDuplicateIdViolation(element, knownElement));
			}
		} else {
			idToElementMap.put(element.getId(), element);
		}

		return result;
	}
}
