/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.storage.eclipse;

import static java.lang.System.identityHashCode;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl;
import org.eclipse.emf.ecore.xml.type.AnyType;
import org.fortiss.tooling.kernel.model.IIdLabeled;

/**
 * {@link org.eclipse.emf.ecore.resource.Resource.Factory} that creates {@link Resource}s using the
 * {@code id} filed of {@link IIdLabeled} model elements.
 * 
 * One effect of using this that {@link EReference}s are encoded based on these IDs,
 * and hence cross-references between multiple resources are more reliable (the
 * fragment-based encoding breaks as soon as the structure of of the referenced
 * model changes).
 * 
 * @author barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: FC2FB6F94F632A4E942D2C5085348710
 */
public class AF3ResourceFactory extends ResourceFactoryImpl {

	/**
	 * {@link XMIResourceImpl} used to persist AF3 models.
	 * 
	 * @author barner
	 * @author $Author$
	 * @version $Rev$
	 * @ConQAT.Rating RED Hash:
	 */
	private final class AF3XMIResource extends XMIResourceImpl {
		/** Constructs a new {@link AF3XMIResource}. */
		private AF3XMIResource(URI uri) {
			super(uri);
		}

		/** {@inheritDoc} */
		@Override
		protected boolean useIDs() {
			return true;
		}

		/** {@inheritDoc} */
		@Override
		public String getID(EObject eObject) {
			String id = super.getID(eObject);
			if(id == null && eObject instanceof IIdLabeled) {
				id = new Integer(((IIdLabeled)eObject).getId()).toString();
				setID(eObject, id);
			}
			return id;
		}

		/** {@inheritDoc} */
		@Override
		public Map<EObject, String> getEObjectToIDMap() {
			if(eObjectToIDMap == null) {
				// See IDENTITY_HASH_COMPARATOR why an alternative map implementation is required
				eObjectToIDMap = new TreeMap<EObject, String>(IDENTITY_HASH_COMPARATOR);
			}

			return eObjectToIDMap;
		}

		// No override is required for getIDToEObjectMap() since its key type is String

		/** {@inheritDoc} */
		@Override
		public Map<EObject, AnyType> getEObjectToExtensionMap() {
			if(eObjectToExtensionMap == null) {
				// See IDENTITY_HASH_COMPARATOR why an alternative map implementation is required.
				eObjectToExtensionMap = new TreeMap<EObject, AnyType>(IDENTITY_HASH_COMPARATOR);
			}
			return eObjectToExtensionMap;
		}
	}

	/**
	 * Comparator based on the {@link System#identityHashCode(Object)} function.
	 * See {@link AF3ResourceFactory#IDENTITY_HASH_COMPARATOR} for more details.
	 * 
	 * @author barner
	 * @author $Author$
	 * @version $Rev$
	 * @ConQAT.Rating RED Hash:
	 */
	private static final class IdentityHashComparator implements Comparator<EObject> {
		/** {@inheritDoc} */
		@Override
		public int compare(EObject o1, EObject o2) {
			Integer h1 = identityHashCode(o1);
			Integer h2 = identityHashCode(o2);
			return h1.compareTo(h2);
		}
	}

	/**
	 * <p>
	 * {@link IdentityHashComparator} used to implement alternative {@link EObject} -> ID /
	 * ExtensionMaps. This is required because {@link XMIResourceImpl} is based on {@link HashMap}s
	 * may not work correctly for keys whose {@link #hashCode()} and {@link #equals(Object)} methods
	 * have been overridden in such a way that different instances of the same object cannot be
	 * distinguished.
	 * </p>
	 * 
	 * <p>
	 * Since it is not possible to provide an alternative {@link #hashCode()} and
	 * {@link #equals(Object)} implementation to a {@link HashMap}, a {@link TreeMap} is used (in
	 * contrast to the implementation in {@link XMIResourceImpl}). Coincidentally, the
	 * implementation {@link IdentityHashComparator#compare(EObject, EObject)} is based on
	 * {@link System#identityHashCode(Object)}.
	 * </p>
	 */
	private static final IdentityHashComparator IDENTITY_HASH_COMPARATOR =
			new IdentityHashComparator();

	/** {@inheritDoc} */
	@Override
	public Resource createResource(URI uri) {
		return new AF3XMIResource(uri);
	}
}
